

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `DAD': Representation of a general array entity.              //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

struct DAD {

  // Constructors

  DAD() {}

  DAD(const int _rank, const Group& _group) : rank(_rank), group(_group) {}

  DAD(const int _rank, const Group& _group, Range* _ranges, Stride* _strides) :
      rank(_rank), group(_group), ranges(_ranges), strides(_strides) {}

  // Data members

  int rank ;

  Group group ;

  Range* ranges  ;

  Stride* strides ;

  // inquiries

  int rnk() const {return rank ; }

  const Group &grp() const {return group ; }

  Range rng(const int r) const {return ranges  [r] ; }

  const Stride &str(const int r) const {return strides [r] ; }

  DimensionSet sig() const {
    DimensionSet signature ;
    for(int r = 0 ; r < rank ; r++)
      signature += ranges [r].dim() ;
    return signature ;
  }
} ;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// `shapeCheck'                                                        //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

inline void shapeCheck(const DAD* a, const DAD* b) {
  if(a->rnk() != b->rnk())
    AdlibError("shapeCheck: Rank mismatch") ;
  for(int i = 0 ; i < a->rnk() ; i++)
    if(a->rng(i).size() != b->rng(i).size())
      AdlibError("shapeCheck: Shape mismatch") ;
}

//// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
////                                                                     //
////  Class Map: blockwise mapping operation.                            //
////                                                                     //
//// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//
//class Map {
//public :
//  Map(const DAD* _arr) : arr(_arr), rank(_arr->rnk()) {
//    exts     = new int [rank] ;
//    step = new int [rank] ;
//  }
//
//  ~Map() {
//    delete [] steps ;
//    delete [] exts ;
//  }
//
//  void execute() {
//    if(arr->grp().member())
//      loop(0, 0, 0) ;
//  }
//
//protected :
//  int rank ;
//  int *exts, *steps ;
//
//  virtual void opMap(const int offset) = 0 ;
//
//private :
//  const DAD* arr ;
//
//  void loop(int offset, const int r) {
//    if(r == rank)
//      opMap(offset) ;
//    else {
//      Stride str = arr->str(r) ;
//
//      LocBlocksIndex i(arr->rng(r)) ;
//      for(i.beginLocBlk() ; i.test() ; i.nextLocBlk()) {
//        exts  [r] = i.count ;
//        steps [r] = str.step(i.sub_stp) ;
//
//        loop(offset + str.offset(i), r + 1) ;
//      }
//    }
//  }
//} ;

//// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
////                                                                     //
////  Class Map2: blockwise mapping operation applied to pair of         //
////              identically distributed arrays.                        //
////                                                                     //
//// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//
//class Map2 {
//public :
//  Map2(const DAD* _dst, const DAD* _src) :
//           dst(_dst), src(_src), rank(_src->rnk()) {
//    shapeCheck(dst, src) ;
//
//    exts     = new int [rank] ;
//    dstSteps = new int [rank] ;
//    srcSteps = new int [rank] ;
//  }
//
//  ~Map2() {
//    delete [] srcSteps ;
//    delete [] dstSteps ;
//    delete [] exts ;
//  }
//
//  void execute() {
//    if(src->grp().member())
//      loop(0, 0, 0) ;
//  }
//
//protected :
//  int rank ;
//  int *exts, *srcSteps, *dstSteps ;
//
//  virtual void opMap(const int dstOffset, const int srcOffset) = 0 ;
//
//private :
//  const DAD* dst ;
//  const DAD* src ;
//
//  void loop(int dstOffset, int srcOffset, const int r) {
//    if(r == rank)
//      opMap(dstOffset, srcOffset) ;
//    else {
//      Stride dstStr = dst->str(r) ;
//      Stride srcStr = src->str(r) ;
//
//      LocBlocksIndex i(dst->rng(r)) ;
//      for(i.beginLocBlk() ; i.test() ; i.nextLocBlk()) {
//        exts     [r] = i.count ;
//        dstSteps [r] = dstStr.step(i.sub_stp) ;
//        srcSteps [r] = srcStr.step(i.sub_stp) ;
//
//        loop(dstOffset + dstStr.offset(i), srcOffset + srcStr.offset(i),
//             r + 1) ;
//      }
//    }
//  }
//} ;


// ----------------------------------------------------------------------
// Things to do
//

